/*
 * Programming in Java, Jacobs University, 2015
 *
 * This class demonstrates the usage of the Callable interface and of
 * Futures, an abstraction to organize concurrent computations which
 * is gaining popularity.
 *
 * $ java C this is some fancy string
 */

import java.lang.Thread;
import java.lang.InterruptedException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.ExecutionException;

public class C implements Callable<String>
{
    private String s = null;
    private int ms = 0;

    public C(String s)
    {
	this(s, 1000);
    }

    public C(String s, int ms)
    {
	this.s = s;
	this.ms = ms;
    }

    public String call()
    {
	String r = "";
	int len = s.length();
	
	for (int i = 0; i < len && ! Thread.currentThread().isInterrupted(); i++) {
	    try {
		TimeUnit.MILLISECONDS.sleep(ms);
	    } catch(InterruptedException e) {
		Thread.currentThread().interrupt();
	    }
	    r += s + " ";
	}
	return r;
    }

    public static void main(String[] args)
	throws InterruptedException, ExecutionException
    {
	ExecutorService pool = Executors.newFixedThreadPool(2);
	List<Future<String>> futures = new ArrayList<Future<String>>();

	for (String arg: args) {
	    futures.add(pool.submit(new C(arg, 123)));
	}

	for (Future<String> f: futures) {
	    System.out.println(f.get());
	}

        pool.shutdown();
    }
}
